/*
This program creates the following pipeline using a Java process
as the parent (it doesn't matter if the child process is written
in Java or C or any other language).

              child                        parent
          +------------+               +------------+
          |            |     pipe      |            |
  stdin--->>0        1>>--0========0--->>0        1>>--->stdout
          |            |               |            |
          |          2>>-->stderr      |          2>>--->stderr
          |            |               |            |
          +------------+               +------------+
          weird_echo.exe           CreateOnePipedChild.class

To set up this pipeline, we need to do three things, create the child
process, have the child inherit its stdin from the parent, and then
connect the parent's stdin to the child's stdout (surprisingly in Java
there is no need to explicitly create a "pipe" object).
*/

import java.io.IOException;
import java.util.Scanner;

public class CreateOnePipedChild
{
   public static void main(String[] args) throws IOException, InterruptedException
   {
      // Create a command line for running the child.
      ProcessBuilder pb1 = new ProcessBuilder("weird_echo.exe");
    //ProcessBuilder pb1 = new ProcessBuilder("cmd.exe", "/C", "start", "weird_echo.exe");
      // Have the child inherit the parent's standard input stream.
      pb1.redirectInput( ProcessBuilder.Redirect.INHERIT );
      // Execute the child.
      Process process = pb1.start();  // this throws IOException

      // Set System.in to be the standard output of the child process.
      // This is a "pipeline" from the child process into the parent.
      System.setIn( process.getInputStream() );

      // Create a Scanner object to make it easier to use System.in.
      // NOTE: System.in is now connected to the child process's stdout.
      //       The parent process has lost access to the keyboard stream
      //      (and the keyboard stream has been redirected to the child's stdin).
      Scanner scanner = new Scanner( System.in );

      // Read and filter each line of data from the child's stdout.
      while ( scanner.hasNextLine() )
      {
         String oneLine = scanner.nextLine(); // read from child's stdout

         // Filter the line of input while printing it.
         for (int i = 0; i < oneLine.length(); i++)
         {
            System.out.print( oneLine.charAt(i) );
            if ( -1 != "aeiouAEIOU".indexOf( oneLine.charAt(i) ) )
            {
               System.out.print( oneLine.charAt(i) ); // double vowels
          }
         }
         System.out.println();
      }

      System.out.println("Child ended.");
      try{Thread.sleep(5000);}catch(Exception e){} // let the parent run for a while
      System.out.println("Parent ended.");

      // Wait for the child to finish its work.
      //process.waitFor();  // this throws InterruptedException
   }
}
